/****************************************/ 
/*            TPR - Counter             */
/*            '''''''''''''             */
/*             19.10.2005               */
/*    CopyLeft 2005 by Jonas Gehring    */
/****************************************/ 

#include <stdio.h>
#include <stdlib.h>

#define CREDITS     "\nTPR - Counter v1.0\nCopyLeft 2005 by Jonas Gehring\n\n"


FILE *inf;
unsigned char tprpath[255];
unsigned char *infile = NULL;
unsigned long long nfiles = 0, nlines = 0, nchars = 0;


// Output: How to use the Program
void PrintUsage(void)
{
    printf(CREDITS);
    printf("Usage: tpr-count <tpr-file>\n\n"                       \
           "                 Counts the total files ,lines and\n"     \
           "                 characters of a TIGCC-project.\n\n"      \
    );
}

int CountStats(void)
{
    unsigned char buffer[65536];
    unsigned char tsrcfile[255];
        
    // Go to the beginning of the file listing
    while (fgets(buffer, 65535, inf)) {
        if (!strncmp(buffer, "[Included Files]", 16)) {
           break;
        }
    }
    
    // Count number of lines for every file
    while (fgets(buffer, 65535, inf)) {
        FILE *srcfile;
        
        buffer[strlen(buffer)-1] = 0;
        unsigned char *srcname = strchr(buffer, '=') + 1;
        
        // Check for source-files
        if (strstr(srcname, ".c") || strstr(srcname, ".h") ||
            strstr(srcname, ".s") || strstr(srcname, ".asm")) {
       
            // Try to open source-file
            if (!strchr(srcname, '\\')) {
               strcpy(tsrcfile, tprpath);
               strcat(tsrcfile, srcname);
            }       
            if ((srcfile = fopen(tsrcfile, "r")) == NULL) {
               printf("\nERROR: unable to open file %s\n", srcname);
               return 0;
            }
            
            //printf("%s\n", tsrcfile);
    
            // Count number of lines for every file
            while (fgets(buffer, 65535, srcfile)) {
               nchars += strlen(buffer);
               nlines ++;
            }
    
            nfiles++;
    
            fclose(srcfile);
        }
        else {
            nfiles++;
        } 
    }
     
    fclose(inf);
    
    return 1;
}


// Program entry point
int main(int argc, char *argv[])
{
    int n;
    unsigned char *tpath;
        
    // Check for arguments
    if (argc < 1) {
        PrintUsage();
        return 0;
    }
    
    // Parse arguments
    for (n = 1; n < argc; n++) {
        if (!infile) {
           infile = argv[n];
        }
        else {
            PrintUsage();
            return 0;
        }
    }
    
    if (!infile) {
        PrintUsage();
        return 0;
    }
    
    // Try to open TPR-file
    if (!(inf = fopen(infile, "r"))) {
        printf("\nERROR: unable to open TPR-file %s\n", infile);
        printf(CREDITS);
        return 0;
    }
    
    // Save path
    strcpy(tprpath, infile);
    while (tprpath[strlen(tprpath)-1] != '\\') {
        tprpath[strlen(tprpath)-1] = 0;   
    }
    
    // Now count the files and lines
    if (CountStats()) {
        // Finally, print the results and the credits
        printf("\nStatistics of %s:\n\n", infile+(strlen(tprpath)));
        printf("Total number of files:                 %lu\n", nfiles);
        printf("Total number of lines in source:       %lu\n", nlines);
        printf("Total number of characters in source:  %lu\n", nchars);
    }
    
    printf(CREDITS);
}
